"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveUri = exports.getFileType = exports.SystemError = exports.systemErrorCode = exports.SystemErrorNo = void 0;
const vscode_uri_1 = require("vscode-uri");
const path_1 = __importDefault(require("path"));
const vscode_1 = require("../vscode");
var SystemErrorNo;
(function (SystemErrorNo) {
    SystemErrorNo["ENOENT"] = "ENOENT";
    SystemErrorNo["EEXIST"] = "EEXIST";
    SystemErrorNo["EISDIR"] = "EISDIR";
})(SystemErrorNo = exports.SystemErrorNo || (exports.SystemErrorNo = {}));
exports.systemErrorCode = {
    [SystemErrorNo.ENOENT]: -2,
    [SystemErrorNo.EEXIST]: -17,
    [SystemErrorNo.EISDIR]: -21,
};
class SystemError extends Error {
    constructor(systemError) {
        super();
        this.code = systemError;
        this.errno = exports.systemErrorCode[systemError];
    }
}
exports.SystemError = SystemError;
const getFileType = (stats) => {
    if (stats.isFile()) {
        return vscode_1.FileType.File;
    }
    if (stats.isDirectory()) {
        return vscode_1.FileType.Directory;
    }
    if (stats.isSymbolicLink()) {
        return vscode_1.FileType.SymbolicLink;
    }
    return vscode_1.FileType.Unknown;
};
exports.getFileType = getFileType;
const resolveUri = (uri, cwd) => path_1.default.join(cwd || process.cwd(), vscode_uri_1.URI.parse(uri).path);
exports.resolveUri = resolveUri;
//# sourceMappingURL=foundation.js.map