"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.textSearch = exports.fileSearch = void 0;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const minimatch_1 = __importDefault(require("minimatch"));
const logger_1 = __importDefault(require("../logger"));
const foundation_1 = require("./foundation");
const _fileSearch = (root, pattern, options, maxResults, cwd) => __awaiter(void 0, void 0, void 0, function* () {
    const { includes, excludes } = options;
    const files = yield fs_1.promises.readdir(foundation_1.resolveUri(root, cwd));
    const result = [];
    for (const file of files) {
        const filePath = path_1.default.join(root, file);
        if ((includes.length && !includes.some((include) => minimatch_1.default(filePath, include))) ||
            excludes.some((exclude) => minimatch_1.default(filePath, exclude))) {
            continue;
        }
        const stats = yield fs_1.promises.stat(foundation_1.resolveUri(filePath, cwd));
        if (stats.isDirectory()) {
            result.push(...(yield _fileSearch(filePath, pattern, options, maxResults && maxResults - result.length, cwd)));
        }
        if (maxResults && result.length >= maxResults) {
            break;
        }
        if (stats.isFile() && filePath.includes(pattern)) {
            result.push(filePath);
        }
        if (maxResults && result.length >= maxResults) {
            break;
        }
    }
    return result;
});
const fileSearch = (_a, cwd) => { var pattern, _b, maxResults, options; return __awaiter(void 0, void 0, void 0, function* () {
    ({ pattern } = _a, _b = _a.options, { maxResults } = _b, options = __rest(_b, ["maxResults"]));
    try {
        return yield _fileSearch(options.folder, pattern, options, maxResults, cwd);
    }
    catch (error) {
        logger_1.default.warn('error when searching file: %s', error);
    }
    return [];
}); };
exports.fileSearch = fileSearch;
const previewMaxPrefix = 20;
const previewMaxLength = 200;
const _textSearch = (root, config, cwd, emitMatch) => __awaiter(void 0, void 0, void 0, function* () {
    const { query, options: { includes, excludes, maxResults, maxFileSize, encoding }, } = config;
    const files = yield fs_1.promises.readdir(foundation_1.resolveUri(root, cwd));
    const result = [];
    let totalResult = 0;
    for (const file of files) {
        const filePath = path_1.default.join(root, file);
        if ((includes.length && !includes.some((include) => minimatch_1.default(filePath, include))) ||
            excludes.some((exclude) => minimatch_1.default(filePath, exclude))) {
            continue;
        }
        const resolvedPath = foundation_1.resolveUri(filePath, cwd);
        const stats = yield fs_1.promises.stat(resolvedPath);
        if (stats.isDirectory()) {
            result.push(...(yield _textSearch(filePath, config, cwd, emitMatch))[0]);
        }
        if (maxResults && totalResult > maxResults) {
            break;
        }
        if (stats.isFile()) {
            if (maxFileSize && stats.size > maxFileSize) {
                continue;
            }
            const content = yield fs_1.promises.readFile(resolvedPath, (encoding !== null && encoding !== void 0 ? encoding : 'utf-8'));
            const lines = content.split('\n');
            const regExp = new RegExp(getPattern(query), getFlags(query));
            const matchResult = {
                path: filePath,
                ranges: [],
                preview: {
                    text: '',
                    matches: [],
                },
            };
            let previewTextLines = 0;
            let startLineIndex = 0;
            let startCharIndex = 0;
            let endLineIndex = 0;
            let endCharIndex = 0;
            let match;
            do {
                match = regExp.exec(content);
                if (match) {
                    totalResult += 1;
                    if (maxResults && totalResult > maxResults) {
                        break;
                    }
                    while (startCharIndex + lines[startLineIndex].length < match.index) {
                        startCharIndex += lines[startLineIndex].length + 1;
                        startLineIndex += 1;
                    }
                    const endIndex = match.index + match[0].length;
                    while (endCharIndex + lines[endLineIndex].length < endIndex) {
                        endCharIndex += lines[endLineIndex].length + 1;
                        endLineIndex += 1;
                    }
                    const startPosition = match.index - startCharIndex;
                    const endPosition = endIndex - endCharIndex;
                    matchResult.ranges.push({
                        startLine: startLineIndex,
                        startPosition,
                        endLine: endLineIndex,
                        endPosition,
                    });
                    // single line preview only
                    let previewText = '';
                    const previewMatch = {
                        startLine: previewTextLines,
                        startPosition,
                        endLine: previewTextLines,
                        endPosition: endLineIndex > startLineIndex ? lines[startLineIndex].length : endPosition,
                    };
                    if (previewMatch.startPosition > previewMaxPrefix) {
                        previewText += lines[startLineIndex].slice(startPosition - previewMaxPrefix) + '\n';
                        previewMatch.endPosition -= startPosition - previewMaxPrefix;
                        previewMatch.startPosition = previewMaxPrefix;
                    }
                    else {
                        previewText += lines[startLineIndex] + '\n';
                    }
                    if (previewText.length > previewMaxLength) {
                        previewText = previewText.slice(0, previewMaxLength) + '\n';
                        previewMatch.endPosition = Math.min(previewMatch.endPosition, previewText.length);
                    }
                    matchResult.preview.matches.push(previewMatch);
                    matchResult.preview.text += previewText;
                    previewTextLines += 1;
                }
            } while (match);
            matchResult.preview.text = matchResult.preview.text.slice(0, -1);
            if (matchResult.ranges.length > 0) {
                result.push(matchResult);
                emitMatch === null || emitMatch === void 0 ? void 0 : emitMatch(matchResult);
            }
        }
        if (maxResults && totalResult > maxResults) {
            break;
        }
    }
    return [result, !!maxResults && totalResult > maxResults, totalResult];
});
const getFlags = ({ isCaseSensitive, isMultiline }) => {
    let result = 'g';
    if (!isCaseSensitive) {
        result += 'i';
    }
    if (isMultiline) {
        result += 'm';
    }
    return result;
};
const getPattern = ({ pattern, isRegExp, isWordMatch }) => {
    let result = pattern;
    if (!isRegExp) {
        // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_Expressions#escaping
        result = result.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
    }
    if (isWordMatch) {
        result = `\\b${result}\\b`;
    }
    return result;
};
const textSearch = (config, cwd, emitMatch) => __awaiter(void 0, void 0, void 0, function* () {
    logger_1.default.verbose('text search with config');
    logger_1.default.verbose(JSON.stringify(config));
    const [, limitHit, totalResult] = yield _textSearch(config.options.folder, config, cwd, emitMatch);
    logger_1.default.verbose('search completed: limitHit=%s totalResult=%s', limitHit, totalResult);
    return { limitHit };
});
exports.textSearch = textSearch;
//# sourceMappingURL=search.js.map