"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerHttpRequestHandlers = void 0;
const core_1 = require("@github-vsc-runner/core");
const http_1 = __importDefault(require("http"));
const logger_1 = __importDefault(require("./logger"));
const requestDict = {};
const registerHttpRequestHandlers = (socket, runner) => {
    socket.on(core_1.VscClientEvent.FetchCurrentPortForwarding, () => {
        socket.emit(core_1.RunnerClientEvent.CurrentPortForwarding, runner.portForwarding);
    });
    socket.on(core_1.VscClientEvent.SetPortForwarding, (port) => {
        logger_1.default.info('port forwarding to %s', port);
        runner.portForwarding = port;
    });
    socket.on(core_1.RunnerServerEvent.HttpRequest, (uuid, type, data) => {
        var _a, _b;
        if (!runner.portForwarding) {
            socket.emit(core_1.RunnerClientEvent.HttpStream, uuid, core_1.RunnerClientHttpStreamType.Error);
            return;
        }
        logger_1.default.verbose('request [%s], uuid=%s', type, uuid);
        logger_1.default.verbose(JSON.stringify(data));
        if (type === core_1.RunnerClientHttpStreamType.Start) {
            const { path, method, headers } = data;
            const clientRequest = http_1.default.request({ host: 'localhost', port: runner.portForwarding, path, method, headers, timeout: 30000 }, (res) => {
                const payload = {
                    status: res.statusCode,
                    headers: res.headers,
                };
                socket.emit(core_1.RunnerClientEvent.HttpStream, uuid, core_1.RunnerClientHttpStreamType.Response, payload);
                res.on('data', (data) => {
                    logger_1.default.debug('client response [data], uuid=%s, length=%s', uuid, data.byteLength);
                    socket.emit(core_1.RunnerClientEvent.HttpStream, uuid, core_1.RunnerClientHttpStreamType.Data, data);
                });
                res.on('end', () => {
                    logger_1.default.verbose('client response [end], uuid=%s', uuid);
                    socket.emit(core_1.RunnerClientEvent.HttpStream, uuid, core_1.RunnerClientHttpStreamType.End);
                    clientRequest.destroy();
                    delete requestDict[uuid];
                });
            });
            clientRequest.on('timeout', () => {
                logger_1.default.verbose('client request [timeout], uuid=%s', uuid);
                clientRequest.end();
            });
            clientRequest.on('error', (error) => {
                logger_1.default.verbose('client request [error], uuid=%s', uuid);
                logger_1.default.verbose(JSON.stringify(error));
                socket.emit(core_1.RunnerClientEvent.HttpStream, uuid, core_1.RunnerClientHttpStreamType.Error, error);
            });
            requestDict[uuid] = clientRequest;
            socket.emit(core_1.RunnerClientEvent.HttpStream, uuid, core_1.RunnerClientHttpStreamType.Start);
        }
        if (type === core_1.RunnerClientHttpStreamType.Data) {
            (_a = requestDict[uuid]) === null || _a === void 0 ? void 0 : _a.write(data);
        }
        if (type === core_1.RunnerClientHttpStreamType.End) {
            (_b = requestDict[uuid]) === null || _b === void 0 ? void 0 : _b.end(data);
        }
    });
};
exports.registerHttpRequestHandlers = registerHttpRequestHandlers;
//# sourceMappingURL=httpProxy.js.map