#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const socket_io_client_1 = require("socket.io-client");
const dotenv_1 = __importDefault(require("dotenv"));
const core_1 = require("@github-vsc-runner/core");
const logger_1 = __importDefault(require("./logger"));
const terminal_1 = require("./terminal");
const fs_1 = require("./fs");
const httpProxy_1 = require("./httpProxy");
dotenv_1.default.config();
const { SERVER_ADDRESS, SESSION_ID, SESSION_OS, GITHUB_WORKSPACE } = process.env;
const cwd = GITHUB_WORKSPACE || process.cwd();
if (!SESSION_ID) {
    logger_1.default.error('missing SESSION_ID from env');
    process.exit(1);
}
logger_1.default.info(`client started with env: SERVER_ADDRESS=${SERVER_ADDRESS} SESSION_ID=***${SESSION_ID.slice(-4)} SESSION_OS=${SESSION_OS}`);
const socket = socket_io_client_1.io(SERVER_ADDRESS || 'wss://localhost:3000', {
    rejectUnauthorized: process.env.NODE_ENV !== 'development',
    transports: ['websocket'],
});
const runner = {
    socket,
    terminals: [],
};
logger_1.default.info('runner client started');
const timeoutHandle = setTimeout(() => {
    logger_1.default.warn('cannot connect to runner server in reasonable time');
    socket.disconnect();
    process.exit(1);
}, 20000);
socket.on('connect', () => {
    clearTimeout(timeoutHandle);
    logger_1.default.info('connected to runner server with id: %s', socket.id);
    socket.emit(core_1.RunnerClientEvent.SetType, SESSION_ID, SESSION_OS);
});
socket.on('disconnect', () => {
    logger_1.default.warn('disconnected');
});
socket.on(core_1.VscClientEvent.TerminateSession, () => {
    logger_1.default.warn('received termination request');
    socket.disconnect();
    process.exit(0);
});
terminal_1.registerTerminalEventHandlers(socket, runner, cwd);
fs_1.registerFSEventHandlers(socket, cwd);
httpProxy_1.registerHttpRequestHandlers(socket, runner);
//# sourceMappingURL=index.js.map