"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerTerminalEventHandlers = exports.closeTerminal = void 0;
const core_1 = require("@github-vsc-runner/core");
const pty = __importStar(require("node-pty"));
const logger_1 = __importDefault(require("./logger"));
const getTerminalById = (runner, terminalId) => {
    const terminal = runner.terminals.find(({ id }) => id === terminalId);
    if (!terminal) {
        logger_1.default.warn('cannot find terminal %s, skipping', terminalId);
    }
    return terminal;
};
const getTerminalPtyProcessById = (runner, terminalId) => {
    const terminal = getTerminalById(runner, terminalId);
    if (!terminal) {
        return;
    }
    if (!terminal.ptyProcess) {
        logger_1.default.warn('no active pty process for terminal id %s, skipping', terminalId);
        return;
    }
    return terminal.ptyProcess;
};
const closeTerminal = (terminal) => {
    terminal.ptyProcess.kill();
};
exports.closeTerminal = closeTerminal;
const registerTerminalEventHandlers = (socket, runner, cwd) => {
    socket.on(core_1.VscClientEvent.ActivateTerminal, (options) => {
        logger_1.default.info('activating terminal with options:');
        logger_1.default.info(JSON.stringify(options, undefined, 2));
        const { id, file, rows, cols } = options;
        const ptyProcess = pty.spawn(file || 'zsh', [], {
            name: 'xterm-color',
            cols: cols || 80,
            rows: rows || 30,
            cwd,
            env: process.env,
        });
        const terminal = Object.assign(Object.assign({}, options), { ptyProcess });
        ptyProcess.onData((data) => {
            logger_1.default.verbose('pty received data', data);
            socket.emit(core_1.RunnerClientEvent.Stdout, id, data);
        });
        ptyProcess.onExit(() => {
            logger_1.default.warn('pty for terminal %s on exit', id);
            runner.terminals = runner.terminals.filter(({ id }) => id !== terminal.id);
            socket.emit(core_1.RunnerClientEvent.TerminalClosed, id);
        });
        runner.terminals.push(terminal);
    });
    socket.on(core_1.VscClientEvent.FetchCurrentTerminals, () => {
        socket.emit(core_1.RunnerClientEvent.CurrentTerminals, runner.terminals.map((_a) => {
            var { ptyProcess } = _a, rest = __rest(_a, ["ptyProcess"]);
            return rest;
        }));
    });
    socket.on(core_1.VscClientEvent.Cmd, (terminalId, data) => {
        logger_1.default.verbose('on vsc client command %s', String(data));
        const ptyProcess = getTerminalPtyProcessById(runner, terminalId);
        if (!ptyProcess) {
            return;
        }
        ptyProcess.write(String(data));
    });
    socket.on(core_1.VscClientEvent.SetTerminalDimensions, (terminalId, { cols, rows }) => {
        const ptyProcess = getTerminalPtyProcessById(runner, terminalId);
        if (!ptyProcess) {
            return;
        }
        logger_1.default.info('setting terminal %s dimensions', terminalId);
        ptyProcess.resize(cols, rows);
    });
    socket.on(core_1.VscClientEvent.CloseTerminal, (terminalId) => {
        const terminal = getTerminalById(runner, terminalId);
        if (!terminal) {
            return;
        }
        logger_1.default.warn('closing terminal %s', terminalId);
        exports.closeTerminal(terminal);
    });
    socket.on(core_1.VscClientEvent.TerminateSession, () => {
        logger_1.default.warn('received termination request');
        socket.disconnect();
        process.exit(0);
    });
};
exports.registerTerminalEventHandlers = registerTerminalEventHandlers;
//# sourceMappingURL=terminal.js.map